"""
[server]
base_path =
redirect_path = /auth_redirect

[notion]
client_id = ...
client_secret = ...

[handler]
consumer = dummy
response_factory = dummy
"""

import configparser

import attr


@attr.s(frozen=True, auto_attribs=True, kw_only=True)
class AppConfiguration:
    consumer_name: str
    response_factory_name: str
    notion_client_id: str = ''
    notion_client_id_key: str = ''
    notion_client_secret: str = ''
    notion_client_secret_key: str = ''
    base_path: str = ''
    redirect_path: str = '/auth_redirect'


def load_config_from_file(filename: str) -> AppConfiguration:
    config = configparser.ConfigParser()
    config.read(filename)
    return AppConfiguration(
        consumer_name=config['handler'].get('consumer', 'dummy'),
        response_factory_name=config['handler'].get('response_factory', 'dummy'),
        notion_client_id=config['notion'].get('client_id', ''),
        notion_client_id_key=config['notion'].get('client_id', ''),
        notion_client_secret=config['notion'].get('client_secret', ''),
        notion_client_secret_key=config['notion'].get('client_secret', ''),
        base_path=config['server'].get('base_path', ''),
        redirect_path=config['server'].get('redirect_path', '/auth_redirect'),
    )



import abc
from http import HTTPStatus
from typing import Any, Optional

from aiohttp.web import Response
from aiohttp.typedefs import LooseHeaders

from notion_oauth_handler.core.dto import TokenResponseInfo


class NotionOAuthResponseFactory(abc.ABC):
    @abc.abstractmethod
    async def make_error_response(self, error: str) -> Response:
        raise NotImplementedError

    @abc.abstractmethod
    async def make_auth_response(self, token_info: TokenResponseInfo) -> Response:
        raise NotImplementedError

    def make_response(
            self,
            *,
            body: Any = None,
            status: int = 200,
            text: Optional[str] = None,
            headers: Optional[LooseHeaders] = None,
            content_type: Optional[str] = None,
    ) -> Response:
        """Just a wrapper for the Response object"""
        return Response(body=body, status=status, text=text, headers=headers, content_type=content_type)


class DummyNotionOAuthResponseFactory(NotionOAuthResponseFactory):
    async def make_error_response(self, error: str) -> Response:
        return self.make_response(text='Error', status=HTTPStatus.FORBIDDEN)

    async def make_auth_response(self, token_info: TokenResponseInfo) -> Response:
        return self.make_response(text='OK', status=HTTPStatus.OK)


class EchoNotionOAuthResponseFactory(NotionOAuthResponseFactory):
    async def make_error_response(self, error: str) -> Response:
        return self.make_response(text=f'Error: {error}', status=HTTPStatus.FORBIDDEN)

    async def make_auth_response(self, token_info: TokenResponseInfo) -> Response:
        return self.make_response(text=f'Token info: {token_info}', status=HTTPStatus.OK)



import os
from importlib import metadata
from typing import Optional, Type

from aiohttp import web

import notion_oauth_handler as package
from notion_oauth_handler.core.consumer import NotionOAuthConsumer
from notion_oauth_handler.server.view import NotionOAuthRedirectView
from notion_oauth_handler.server.middleware import notion_oauth_middleware_factory
from notion_oauth_handler.server.response import NotionOAuthResponseFactory
from notion_oauth_handler.server.config import AppConfiguration, load_config_from_file


def _get_consumer_from_name(consumer_name: str) -> NotionOAuthConsumer:
    entrypoints = metadata.entry_points()[f'{package.__name__}.consumer']
    consumer_cls: Type[NotionOAuthConsumer] = next(
        iter(ep for ep in entrypoints if ep.name == consumer_name)
    ).load()
    consumer = consumer_cls()
    return consumer


def _get_response_factory_from_name(response_factory_name: str) -> NotionOAuthResponseFactory:
    entrypoints = metadata.entry_points()[f'{package.__name__}.response_factory']
    response_factory_cls: Type[NotionOAuthResponseFactory] = next(
        iter(ep for ep in entrypoints if ep.name == response_factory_name)
    ).load()
    response_factory = response_factory_cls()
    return response_factory


def make_app(
        consumer: NotionOAuthConsumer,
        response_factory: NotionOAuthResponseFactory,
        notion_client_id: str,
        notion_client_secret: str,
        base_path: str = '',
        redirect_path: str = '/auth_redirect',
) -> web.Application:
    """
    Create Notion OAuth handling server (aiohttp Application)

    :param consumer: a `NotionOAuthConsumer` subclass instance
        that implements the main logic (e.g. stores the token in a database)
    :param response_factory: a `NotionOAuthResponseFactory` subclass instance
        that renders HTTP responses of the server
    :param notion_client_id: client ID of the Notion integration
    :param notion_client_secret: client secret of the Notion integration
    :param base_path: custom base path for all routes
    :param redirect_path: relative path for the redirect handler
    :return: an aiohttp `Application` instance
    """

    base_path = base_path.rstrip('/')
    redirect_path = redirect_path.lstrip('/')

    app = web.Application(
        middlewares=[
            notion_oauth_middleware_factory(
                consumer=consumer,
                response_factory=response_factory,
                notion_client_id=notion_client_id,
                notion_client_secret=notion_client_secret,
            )
        ],
    )
    app.add_routes([
        web.get(f'{base_path}/{redirect_path}', NotionOAuthRedirectView),
    ])
    return app


def make_app_from_config(config: AppConfiguration) -> web.Application:
    if config.notion_client_id:
        notion_client_id = config.notion_client_id
    else:
        notion_client_id = os.environ[config.notion_client_id_key]

    if config.notion_client_secret:
        notion_client_secret = config.notion_client_secret
    else:
        notion_client_secret = os.environ[config.notion_client_secret_key]

    return make_app(
        consumer=_get_consumer_from_name(config.consumer_name),
        response_factory=_get_response_factory_from_name(config.response_factory_name),
        notion_client_id=notion_client_id,
        notion_client_secret=notion_client_secret,
        base_path=config.base_path,
        redirect_path=config.redirect_path,
    )


def make_app_from_file(filename: Optional[str] = None) -> web.Application:
    if not filename:
        filename = os.environ.get('NOTION_OUATH_HANDLER_CONFIG', 'notion-oauth-handler.ini')
    assert filename is not None
    config = load_config_from_file(filename)
    return make_app_from_config(config=config)


async def make_app_from_file_async(filename: Optional[str] = None) -> web.Application:
    return make_app_from_file(filename)



from typing import Any

import attr


@attr.s(frozen=True, auto_attribs=True, kw_only=True)
class AuthRedirectInfo:
    redirect_uri: str
    code: str
    state: str


@attr.s(frozen=True, auto_attribs=True, kw_only=True)
class TokenResponseInfo:
    access_token: str
    workspace_id: str
    workspace_name: str
    workspace_icon: str
    bot_id: str
    owner: dict[str, Any]
